import 'package:flutter/material.dart';
import 'dart:ui' as ui;

class VidPainter extends CustomPainter {
  VidPainter(
      {required this.image,
      required this.frameIndex,
      this.colorFilterMatrix = const <double>[
        1, 0, 0, 0, 0, // red
        0, 1, 0, 0, 0, // green
        0, 0, 1, 0, 0, // blue
        0, 0, 0, 1, 0, // alpha
      ],
      this.enableScale = true});
  ui.Image image;
  int frameIndex;
  List<double> colorFilterMatrix;

  /// 允许自适应缩放
  bool enableScale;

  @override
  void paint(Canvas canvas, Size size) async {
    canvas.save();

    /// TODO: 自适应缩放只做了横向的，纵向的还没有做
    if (enableScale) {
      final double scale = size.width / image.width;
      final double offsetY = (size.height - image.height * scale) / 2;
      canvas.drawImageRect(
          image,
          Rect.fromLTWH(0, 0, image.width.toDouble(), image.height.toDouble()),
          Rect.fromLTWH(0, offsetY, size.width, image.height * scale),
          Paint()..colorFilter = ColorFilter.matrix(colorFilterMatrix));
    } else {
      final double offsetX = (size.width - image.width) / 2;
      final double offsetY = (size.height - image.height) / 2;
      canvas.drawImageRect(
          image,
          Rect.fromLTWH(0, 0, image.width.toDouble(), image.height.toDouble()),
          Rect.fromLTWH(offsetX, offsetY, image.width.toDouble(),
              image.height.toDouble()),
          Paint()..colorFilter = ColorFilter.matrix(colorFilterMatrix));
    }
    canvas.restore();
  }

  @override
  bool shouldRepaint(covariant VidPainter oldDelegate) {
    return oldDelegate.frameIndex != frameIndex ||
        oldDelegate.colorFilterMatrix != colorFilterMatrix;
  }
}
